@@ -4,6 +4,7 @@ import unittest
 from unittest import mock
 
 import asyncio
+from types import GenericAlias
 from test.test_asyncio import utils as test_utils
 
 
@@ -92,6 +93,11 @@ class QueueBasicTests(_QueueTestBase):
     def test_str(self):
         self._test_repr_or_str(str, False)
 
+    def test_generic_alias(self):
+        q = asyncio.Queue[int]
+        self.assertEqual(q.__args__, (int,))
+        self.assertIsInstance(q, GenericAlias)
+
     def test_empty(self):
         with self.assertWarns(DeprecationWarning):
             q = asyncio.Queue(loop=self.loop)
